/*

reinhard@finalmedia.de
Do 16. Mär 13:14:59 CET 2023

read current framebuffer width, height and
colordepth via fnctl.

set (overwrite) environment variables
SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_COLORDEPTH
and fork given child process

Usage Example: fbenv fbdraw < testdata.txt

Default framebuffer device is /dev/fb0
to use another device, use

export FBDEV=/dev/fb1
fbenv

*/

#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include <err.h>
#include <string.h>

#define FBIOGET_VSCREENINFO	0x4600
#define FBIOGET_FSCREENINFO	0x4602

struct fb_bitfield {
	uint32_t offset;		/* beginning of bitfield	*/
	uint32_t length;		/* length of bitfield		*/
	uint32_t msb_right;		/* != 0 : Most significant bit is */
					/* right */
};

struct fb_var_screeninfo {
	uint32_t xres;			/* visible resolution		*/
	uint32_t yres;
	uint32_t xres_virtual;		/* virtual resolution		*/
	uint32_t yres_virtual;
	uint32_t xoffset;		/* offset from virtual to visible */
	uint32_t yoffset;		/* resolution			*/
	uint32_t bits_per_pixel;	/* guess what			*/
	uint32_t grayscale;		/* 0 = color, 1 = grayscale,	*/
					/* >1 = FOURCC			*/
	struct fb_bitfield red;		/* bitfield in fb mem if true color, */
	struct fb_bitfield green;	/* else only length is significant */
	struct fb_bitfield blue;
	struct fb_bitfield transp;	/* transparency			*/

	uint32_t nonstd;		/* != 0 Non standard pixel format */

	uint32_t activate;		/* see FB_ACTIVATE_*		*/

	uint32_t height;		/* height of picture in mm    */
	uint32_t width;			/* width of picture in mm     */

	uint32_t accel_flags;		/* (OBSOLETE) see fb_info.flags */

	uint32_t pixclock;		/* pixel clock in ps (pico seconds) */
	uint32_t left_margin;		/* time from sync to picture	*/
	uint32_t right_margin;		/* time from picture to sync	*/
	uint32_t upper_margin;		/* time from sync to picture	*/
	uint32_t lower_margin;
	uint32_t hsync_len;		/* length of horizontal sync	*/
	uint32_t vsync_len;		/* length of vertical sync	*/
	uint32_t sync;			/* see FB_SYNC_*		*/
	uint32_t vmode;			/* see FB_VMODE_*		*/
	uint32_t rotate;		/* angle we rotate counter clockwise */
	uint32_t colorspace;		/* colorspace for FOURCC-based modes */
	uint32_t reserved[4];		/* Reserved for future compatibility */
};

struct fb_fix_screeninfo {
	char id[16];			/* identification string eg "TT Builtin" */
	unsigned long smem_start;	/* Start of frame buffer mem */
					/* (physical address) */
	uint32_t smem_len;			/* Length of frame buffer mem */
	uint32_t type;			/* see FB_TYPE_*		*/
	uint32_t type_au;			/* Interleave for interleaved Planes */
	uint32_t visual;		/* see FB_VISUAL_*		*/
	uint16_t xpanstep;		/* zero if no hardware panning  */
	uint16_t ypanstep;		/* zero if no hardware panning  */
	uint16_t ywrapstep;		/* zero if no hardware ywrap    */
	uint32_t line_length;		/* length of a line in bytes    */
	unsigned long mmio_start;	/* Start of Memory Mapped I/O   */
					/* (physical address) */
	uint32_t mmio_len;			/* Length of Memory Mapped I/O  */
	uint32_t accel;			/* Indicate to driver which	*/
					/*  specific chip/card we have	*/
	uint16_t capabilities;		/* see FB_CAP_*			*/
	uint16_t reserved[2];		/* Reserved for future compatibility */
};



int main(int argc, char *argv[]) {

	if (argc<2) errx(64, "Usage: %s command [args ..]",argv[0]);

	// default to fb0, if env not set
	setenv("FBDEV","/dev/fb0",0);

	int fbfd = 0;
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;

	char swidth[65];
	char sheight[65];
 	char scolordepth[65];

	// Open the file for reading and writing
	fbfd = open(getenv("FBDEV"), O_RDWR);

 	if (fbfd == -1) err(1,"Error opening framebuffer device");
 	if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo) == -1) err(2,"Error reading fixed information");
 	if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo) == -1) err(3,"Error reading variable information");

	close(fbfd);

 	snprintf(swidth,64,"SCREEN_WIDTH=%d",(int) vinfo.xres);
 	snprintf(sheight,64,"SCREEN_HEIGHT=%d",(int) vinfo.yres);
 	snprintf(scolordepth,64,"SCREEN_COLORDEPTH=%d",(int) vinfo.bits_per_pixel);

	putenv(swidth);
	putenv(sheight);
	putenv(scolordepth);

	argv++;
	execvp(*argv, argv);
	err(4,"Error executing child process '%s'", *argv);

	return 0;
}
